/**
*	vector.cpp
*	CS 6610 - Advanced Computer Graphics I
*	Zach Gildersleeve
*	Oct. 13, 2006
*	
*	A bunch of vector and 3D math functions.  Adapted from GLT toolset from
*	"OpenGL Superbible", and various places on the internet.
*/

#include "main.h"
#include "vector.h"

/**
*	Subtracts two vectors, and stores the result in a third.
*	@param	vFirst		a vector
*	@param	vSecond		the vector to subtract from vFirst
*	@param	vResult		the vector to store the result in
*/
void v_subtractVectors(const vector3 vFirst, const vector3 vSecond, vector3 vResult) 
{
    vResult[0] = vFirst[0] - vSecond[0];
    vResult[1] = vFirst[1] - vSecond[1];
    vResult[2] = vFirst[2] - vSecond[2];
}

/**
*	Gets the dot product between two vectors.
*	@param	vU		the first vector
*	@param	vV		the second vector
*	@return		the dot product of the two vectors
*/
GLfloat v_dotProduct(const vector3 vU, const vector3 vV)
{
    return vU[0]*vV[0] + vU[1]*vV[1] + vU[2]*vV[2]; 
}

/**
*	Gets the length of a vector.
*	@param	v		the vector
*	@return			the length of the vector
*/
GLfloat v_getVectorLength(const vector3 v)
{ 
    return (GLfloat)sqrt(v_getVectorLengthSqrd(v)); 
}

/**
*	Calculates the cross product of two vectors, and stores the result in the third.
*	@param	vU			the first vector
*	@param	vV			the second vector
*	@param	vResult		the resulting vector
*/
void v_crossProduct(const vector3 vU, const vector3 vV, vector3 vResult)
{
	vResult[0] = vU[1]*vV[2] - vV[1]*vU[2];
	vResult[1] = -vU[0]*vV[2] + vV[0]*vU[2];
	vResult[2] = vU[0]*vV[1] - vV[0]*vU[1];
}
/**
*	Normalizes a vector.
*	@param	v	the vector to normalize
*/
void v_normalize(vector3 v)
{ 
    GLfloat length = 1.0f / v_getVectorLength(v);
    v_scaleVector(v, length); 
}

/**
*	Scales a vector.
*	@param	v	the vector to normalize
*	@param	scale	the amount to scale the vector by
*/
void v_scaleVector(vector3 v, const GLfloat scale)
{ 
    v[0] *= scale; v[1] *= scale; v[2] *= scale; 
}

/**
*	Gets the vector length squared.
*	@param	v	the vector
*	@return		the vector length squared
*/
GLfloat v_getVectorLengthSqrd(const vector3 v)
{ 
    return (v[0]*v[0]) + (v[1]*v[1]) + (v[2]*v[2]); 
}

/**
*	Calculates the normal of the three given points, in counter-clockwise order.
*	@param	vP1		the first vector
*	@param	vP2		the second vector
*	@param	vP3		the third vector
*	@param	vNormal	the resulting normal
*/
void v_getNormal(const vector3 vP1, const vector3 vP2, const vector3 vP3, vector3 vNormal)
{
    vector3 vV1, vV2;
    
    v_subtractVectors(vP2, vP1, vV1);
    v_subtractVectors(vP3, vP1, vV2);
    
    v_crossProduct(vV1, vV2, vNormal);
    v_normalize(vNormal);
}

float v_distance(const vector3 v1, const vector3 v2)
{
	float dx, dy, dz;
	
	dx = v1[0] - v2[0];
	dy = v1[1] - v2[1];
	dz = v1[2] - v2[2];
	
	return 	sqrt(dx * dx + dy * dy + dz * dz);
}

