uniform sampler2D edgeTexture;
uniform sampler2D colorTexture;
uniform float threshold;

void main(void)
{
	const float offset = 1.0 / 512.0;
	vec2 texCoord = gl_TexCoord[0].xy;
	
	// Texture Offset Matrix
	//		tl  t	tr
	//		 l	c	r
	//		bl	b	br
	
	vec4 c  = texture2D(edgeTexture, texCoord);
	vec4 bl = texture2D(edgeTexture, texCoord + vec2(-offset, -offset));
	vec4 l  = texture2D(edgeTexture, texCoord + vec2(-offset,     0.0));
	vec4 tl = texture2D(edgeTexture, texCoord + vec2(-offset,  offset));
	vec4 t  = texture2D(edgeTexture, texCoord + vec2(    0.0,  offset));
	vec4 tr = texture2D(edgeTexture, texCoord + vec2( offset,  offset));
	vec4 r  = texture2D(edgeTexture, texCoord + vec2( offset,     0.0));
	vec4 br = texture2D(edgeTexture, texCoord + vec2( offset,  offset));
	vec4 b  = texture2D(edgeTexture, texCoord + vec2(    0.0, -offset));

	// Sobel Edge Detection convolution masks
	// x = -1  0 +1			y = -1 -2 -1
	//     -2  0 +2				 0  0  0
	//     -1  0 +1				+1 +2 +1
	
	vec4 x = -(tl + 2.0 * l + bl) + (tr + 2.0 * r + br);
	vec4 y = -(tl + 2.0 * t + tr) + (bl + 2.0 * b + br);
	
	vec4 pixel = sqrt(x * x + y * y);
	
	if (pixel.x >= threshold || pixel.y >= threshold || pixel.z >= threshold)
		{ gl_FragColor = vec4(0.0);}
	else
		{gl_FragColor = texture2D(colorTexture, texCoord);}
}