#include "./include/main.h"

int cur_button = -1;					// mouse handling for main window
int last_x;								// mouse position
int last_y;								// mouse position
float eye[3];							// camera position
float lookat[3];						// camera look position
int main_window;						// the main window name
int viewX = 209, viewY, viewW, viewH;	// variables to builds the viewport around the GLUI GUI

GLUI				*glui;				// pointers for all of the glui controls
GLUI_RadioGroup		*borderRadio;
GLUI_RadioGroup		*lightRadio;
GLUI_RadioGroup		*edgeRadio;
GLUI_Rollout		*borderRollout;
GLUI_Rollout		*lightRollout;
GLUI_Panel			*lightPanel;
GLUI_Panel			*borderPanel;
GLUI_Spinner		*borderSpinner;
GLUI_Spinner		*thresholdSpinner;
GLUI_Spinner		*blue1Spinner;
GLUI_Spinner		*blue2Spinner;
GLUI_Spinner		*yellow1Spinner;
GLUI_Spinner		*yellow2Spinner;

float blue1		= 0.4;			// relative blue hue
float blue2		= 0.4;			// relative blue value
float yellow1	= 0.2;			// relative yellow hue
float yellow2	= 0.6;			// relative yellow value

GLuint edgeTex;			// texture for hardware edges
GLuint colorTex;				// texture for hardware fill

GLfloat  diffuseLight[] = {1.0, 1.0, 1.0, 1.0};		// color of overall ambient light
GLfloat	 lightPos[]		= {-10.0, 10.0, 0.0, 1.0};	// position of light
float lightIntensity	= 1.0;						// controls intensity of scene light

int silhouette			= 0;	// switch for image space silhouette mode
int edge				= 0;	// switch for hardware silhouette mode
int lightMode			= 0;	// switch for lighting model
int borderThickness		= 1.0;	// switch for image space silhouette thickness
float borderThreshold	= 1.0;	// switch for hardware edge detection threshold

GLenum diffuseGLSL;				// names for GLSL shader program elements
GLenum diffuseVertex;
GLenum diffuseFragment;
GLenum toneGLSL;
GLenum toneVertex;
GLenum toneFragment;
GLenum edgeVertex;
GLenum edgeFragment;
GLenum edgeGLSL;
GLenum normalVertex;
GLenum normalFragment;
GLenum normalGLSL;

/**
*	Idle function for GLUT callback
*/
void myGlutIdle(void)
{
	if (glutGetWindow() != main_window)
		glutSetWindow(main_window);
				
	glutPostRedisplay();
}

/**
*	Mouse callback function for GLUT
*
*	@param	button	the current mouse button
*	@param	state	the state of 'button'
*	@param	x		the x coord of the mouse
*	@param	y		the y coord of the mouse
*/
void myGlutMouse(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN)
		cur_button = button;
	else
	{
		if (button == cur_button)
			cur_button = -1;
	}
	
	last_x = x;
	last_y = y;
}

/**
*	Mouse motion callback function for GLUT
*
*	@param	x		the x coord of the mouse
*	@param	y		the y coord of the mouse
*/
void myGlutMotion(int x, int y)
{
	int dx = x-last_x;			// change in mouse position
	int dy = y-last_y;			// change in mouse position
	
	float scale, len, theta;	// controls camera movement
	float neye[3], neye2[3];	// temp eye positions
	float f[3], r[3], u[3];		// more camera coords
	
	switch(cur_button)
	{
		case GLUT_LEFT_BUTTON:	// translate
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			
			scale = sqrt(v_getVectorLength(f)) * 0.007;	// scale the change by how far away we are
			
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			v_normalize(r);
			v_normalize(u);
			
			eye[0] += -r[0]*dx*scale + u[0]*dy*scale;
			eye[1] += -r[1]*dx*scale + u[1]*dy*scale;
			eye[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			lookat[0] += -r[0]*dx*scale + u[0]*dy*scale;
			lookat[1] += -r[1]*dx*scale + u[1]*dy*scale;
			lookat[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			break;
			
		case GLUT_MIDDLE_BUTTON:	// zoom
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			
			len = v_getVectorLength(f);
			v_normalize(f);
			
			len -= sqrt(len)*dx*0.03;	// scale the change by how far away we are
			
			eye[0] = lookat[0] - len*f[0];
			eye[1] = lookat[1] - len*f[1];
			eye[2] = lookat[2] - len*f[2];
			
			// make sure the eye and lookat points are sufficiently far away
			// push the lookat point forward if it is too close
			if (len < 1)
			{
				lookat[0] = eye[0] + f[0];
				lookat[1] = eye[1] + f[1];
				lookat[2] = eye[2] + f[2];
			}
				break;
			
		case GLUT_RIGHT_BUTTON:	// rotate
			
			neye[0] = eye[0] - lookat[0];
			neye[1] = eye[1] - lookat[1];
			neye[2] = eye[2] - lookat[2];
			
			// first rotate in the x/z plane
			theta = -dx * 0.007;
			neye2[0] = (float)cos(theta)*neye[0] + (float)sin(theta)*neye[2];
			neye2[1] = neye[1];
			neye2[2] =-(float)sin(theta)*neye[0] + (float)cos(theta)*neye[2];
			
			// now rotate vertically
			theta = -dy * 0.007;
			
			f[0] = -neye2[0];
			f[1] = -neye2[1];
			f[2] = -neye2[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			len = v_getVectorLength(f);
			v_normalize(f);
			v_normalize(u);
			
			neye[0] = len * ((float)cos(theta)*f[0] + (float)sin(theta)*u[0]);
			neye[1] = len * ((float)cos(theta)*f[1] + (float)sin(theta)*u[1]);
			neye[2] = len * ((float)cos(theta)*f[2] + (float)sin(theta)*u[2]);
			
			eye[0] = lookat[0] - neye[0];
			eye[1] = lookat[1] - neye[1];
			eye[2] = lookat[2] - neye[2];
			
			break;
	}
	
	last_x = x;
	last_y = y;
	
	glutPostRedisplay();
}

/**
*	Reshape callback function for GLUT
*
*	@param	x	the x window coord - unused
*	@param	y	the y window coord - unused
*/
void myGlutReshape(int	x, int y)
{
	GLUI_Master.get_viewport_area(&viewX, &viewY, &viewW, &viewH);
	glViewport(viewX, viewY, viewW, viewH);	
}

/**
*	Loads the source code of a shader program by
*	copying the entire program into a single string.
*
*	@param	filename	the filename and location of the source code
*	@return		a pointer to the built string
*/
const GLcharARB* loadSource(const char* filename) 
{
	std::string buf;
	std::string line;
	
	std::ifstream in(filename);
	
	if(!in) {
		fprintf(stderr, "Could not find file %s\n", filename);
		exit(EXIT_FAILURE);
	}
	
	while(std::getline(in,line)) {
		buf += line; buf += '\n';
	}
	
	GLcharARB* ret = (GLcharARB*)malloc(buf.size()+1);
	strncpy(ret, buf.c_str(), buf.size());
	ret[buf.size()] = '\0';
	
	return ret;
}

/**
*	Loads the shader source code, compiles the seperate elements,
*	links the vertex and fragment shaders to the GLSL program,
*	and compiles the GLSL program.
*/
void createShaders()
{
	const GLcharARB* diffuseVertexSource;
	const GLcharARB* diffuseFragmentSource;
	const GLcharARB* toneVertexSource;
	const GLcharARB* toneFragmentSource;
	const GLcharARB* edgeVertexSource;
	const GLcharARB* edgeFragmentSource;
	const GLcharARB* normalVertexSource;
	const GLcharARB* normalFragmentSource;
	GLint status;

	diffuseGLSL		= glCreateProgramObjectARB();
	diffuseVertex	= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	diffuseFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	
	toneGLSL		= glCreateProgramObjectARB();
	toneVertex		= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	toneFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);

	edgeGLSL		= glCreateProgramObjectARB();
	edgeVertex		= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	edgeFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);	
	
	normalGLSL		= glCreateProgramObjectARB();
	normalVertex	= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	normalFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);	
	
	diffuseVertexSource		= loadSource("./diffuseVertex.vert");
	diffuseFragmentSource	= loadSource("./diffuseFragment.frag");
	toneVertexSource		= loadSource("./toneVertex.vert");
	toneFragmentSource		= loadSource("./toneFragment.frag");
	edgeVertexSource		= loadSource("./edgeVertex.vert");
	edgeFragmentSource		= loadSource("./edgeFragment.frag");
	normalVertexSource		= loadSource("./normalVertex.vert");
	normalFragmentSource	= loadSource("./normalFragment.frag");

	glShaderSourceARB(diffuseVertex,	1, &diffuseVertexSource, NULL);
	glShaderSourceARB(diffuseFragment,	1, &diffuseFragmentSource, NULL);
	glShaderSourceARB(toneVertex,		1, &toneVertexSource, NULL);
	glShaderSourceARB(toneFragment,		1, &toneFragmentSource, NULL);
	glShaderSourceARB(edgeVertex,		1, &edgeVertexSource, NULL);
	glShaderSourceARB(edgeFragment,		1, &edgeFragmentSource, NULL);
	glShaderSourceARB(normalVertex,		1, &normalVertexSource, NULL);
	glShaderSourceARB(normalFragment,	1, &normalFragmentSource, NULL);

	delete diffuseVertexSource;
	delete diffuseFragmentSource;
	delete toneVertexSource;
	delete toneFragmentSource;
	delete edgeVertexSource;
	delete edgeFragmentSource;
	delete normalVertexSource;
	delete normalFragmentSource;
	
	glCompileShaderARB(diffuseVertex);
	glGetObjectParameterivARB(diffuseVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(diffuseVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(diffuseVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(diffuseFragment);
	glGetObjectParameterivARB(diffuseFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(diffuseFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(diffuseFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(diffuseGLSL, diffuseVertex);
	glAttachObjectARB(diffuseGLSL, diffuseFragment);
	
	glLinkProgramARB(diffuseGLSL);
	glGetObjectParameterivARB(diffuseGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(diffuseGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(diffuseGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	glCompileShaderARB(toneVertex);
	glGetObjectParameterivARB(toneVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(toneVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(toneVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(toneFragment);
	glGetObjectParameterivARB(toneFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(toneFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(toneFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(toneGLSL, toneVertex);
	glAttachObjectARB(toneGLSL, toneFragment);
	
	glLinkProgramARB(toneGLSL);
	glGetObjectParameterivARB(toneGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(toneGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(toneGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	glCompileShaderARB(edgeVertex);
	glGetObjectParameterivARB(edgeVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(edgeVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(edgeVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(edgeFragment);
	glGetObjectParameterivARB(edgeFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(edgeFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(edgeFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(edgeGLSL, edgeVertex);
	glAttachObjectARB(edgeGLSL, edgeFragment);
	
	glLinkProgramARB(edgeGLSL);
	glGetObjectParameterivARB(edgeGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(edgeGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(edgeGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}

	glCompileShaderARB(normalVertex);
	glGetObjectParameterivARB(normalVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(normalVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(normalVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(normalFragment);
	glGetObjectParameterivARB(normalFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(normalFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(normalFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(normalGLSL, normalVertex);
	glAttachObjectARB(normalGLSL, normalFragment);
	
	glLinkProgramARB(normalGLSL);
	glGetObjectParameterivARB(normalGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(normalGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(normalGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
}

/**
*	Initializes the OpenGL state and loads the texture units.
*/
GLvoid Init(GLvoid)
{
	GLfloat  ambientLight[]			= {0.1, 0.1, 0.1, 0.0};		// color of overall ambient light

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	glEnable(GL_LINE_SMOOTH);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_CULL_FACE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glEnable(GL_LIGHT0);

	glClearColor(0.5, 0.5, 0.5, 0.0);
	glClearDepth(1.0);

	glGenTextures(1, &edgeTex);
	glBindTexture(GL_TEXTURE_2D, edgeTex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WINDOW_WIDTH, WINDOW_HEIGHT, 0, GL_RGB, GL_FLOAT, NULL);

	glGenTextures(1, &colorTex);
	glBindTexture(GL_TEXTURE_2D, colorTex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WINDOW_WIDTH, WINDOW_HEIGHT, 0, GL_RGB, GL_FLOAT, NULL);
	
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, edgeTex);
	
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, colorTex);
}

/**
*	Draws the objects with various colors.
*/
GLvoid drawObjects(GLvoid)
{
	GLfloat yellow[]		= {0.8, 0.8, 0.2, 1.0};
	GLfloat green[]			= {0.2, 0.8, 0.2, 1.0};
	GLfloat red[]			= {0.8, 0.2, 0.2, 1.0};
	GLfloat white[]			= {1.0, 1.0, 1.0, 1.0};

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, yellow);

	glPushMatrix();
		glTranslatef(0.0, -2.6, 0.0);
		glScalef(2.0, 2.0, 2.0);
		glRotatef(45.0, 0.0, 1.0, 0.0);

		glutSolidTorus(0.2, 1.0, 50, 50);
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, green);
		glutSolidTorus(0.2, 1.0, 50, 50);
	
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, red);
		glutSolidTorus(0.2, 1.0, 50, 50);
		
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, white);
		glutSolidTorus(0.2, 1.0, 50, 50);
	glPopMatrix();

	glPushMatrix();
		glTranslatef(6.0, 0.0, 0.0);
		glScalef(3.0, 3.0, 3.0);

		glutSolidSphere(1.0, 60.0, 60.0);
		
		glTranslatef(-4.0, 0.0, 0.0);
		glutSolidIcosahedron();
	glPopMatrix();
}

/**
*	Draws the objects white, and applys the normal
*	shader to the objects.
*/
GLvoid drawObjectsForNormals(GLvoid)
{
	glUseProgramObjectARB(normalGLSL);

	glPushMatrix();
		glTranslatef(0.0, -2.6, 0.0);
		glScalef(2.0, 2.0, 2.0);
		glRotatef(45.0, 0.0, 1.0, 0.0);
		
		glutSolidTorus(0.2, 1.0, 50, 50);
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glutSolidTorus(0.2, 1.0, 50, 50);
		
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glutSolidTorus(0.2, 1.0, 50, 50);
		
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glutSolidTorus(0.2, 1.0, 50, 50);
	glPopMatrix();
		
	glPushMatrix();
		glTranslatef(6.0, 0.0, 0.0);
		glScalef(3.0, 3.0, 3.0);	
		glutSolidSphere(1.0, 60.0, 60.0);
		
		glTranslatef(-4.0, 0.0, 0.0);
		glutSolidIcosahedron();
	glPopMatrix();

	glUseProgramObjectARB(0);
}

/**
*	Initializes the uniform variables for the shader programs.
*/
GLvoid setUniformVars(GLvoid)
{
	glUseProgramObjectARB(toneGLSL);
	
		GLint handle = glGetUniformLocationARB(toneGLSL, "blueShift");
			glUniform1fARB(handle, blue1);
	
		handle = glGetUniformLocationARB(toneGLSL, "blueLuminance");
			glUniform1fARB(handle, blue2);
	
		handle = glGetUniformLocationARB(toneGLSL, "yellowShift");
			glUniform1fARB(handle, yellow1);

		handle = glGetUniformLocationARB(toneGLSL, "yellowLuminance");
			glUniform1fARB(handle, yellow2);

	glUseProgramObjectARB(edgeGLSL);

		handle = glGetUniformLocationARB(edgeGLSL, "threshold");
			glUniform1fARB(handle, borderThreshold);
	
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, edgeTex);
		handle = glGetUniformLocationARB(edgeGLSL, "edgeTexture");
			glUniform1iARB(handle, 0);
	
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, colorTex);	
		handle = glGetUniformLocationARB(edgeGLSL, "colorTexture");
			glUniform1iARB(handle, 1);
	
	glUseProgramObjectARB(0);
}

/**
*	Draws a full sized quad to force shader computation
*	or to display full screen results of rendering.
*/
GLvoid drawFullsizeQuad(GLvoid)
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
		glLoadIdentity();
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();
	
		glBegin(GL_QUADS);
			glTexCoord2f(0.0, 0.0);		glVertex3f(-1.0, -1.0, 0.0);
			glTexCoord2f(1.0, 0.0);		glVertex3f(1.0, -1.0, 0.0);
			glTexCoord2f(1.0, 1.0);		glVertex3f(1.0, 1.0, 0.0);
			glTexCoord2f(0.0, 1.0);		glVertex3f(-1.0, 1.0, 0.0);
		glEnd();
	glPopMatrix();
	
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

/**
*	Draws multiple instances of the geometry, depending on the
*	desired render status.  See report for more details.
*/
void Display(void)
{	
	vector3 eyePos;		// position of camera
	float eyeDistance = v_distance(eye, lookat);	// distance from the camera to the lookat point	
	float k = 0.01 * borderThickness;				// thickness of silhouette
	
	eyePos[0] = eye[0];
	eyePos[1] = eye[1];
	eyePos[2] = eye[2];
	v_normalize(eyePos);

	// set up projection and modelview matrix
	diffuseLight[0] = diffuseLight[1] = diffuseLight[2] = lightIntensity;
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluPerspective(70.0, 1.0, 1.0, 40.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	
	gluLookAt(eye[0], eye[1], eye[2], lookat[0], lookat[1], lookat[2], 0, 1, 0);


	setUniformVars();

	// draws the objects initially to fill colorTex texture
	// uses lighting model selected
		if (lightMode == 0)
			glUseProgramObjectARB(diffuseGLSL);
		else
			glUseProgramObjectARB(toneGLSL);

		drawObjects();

		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, colorTex);
		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);

		glUseProgramObjectARB(0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (edge != 0)
	{
		silhouette = 2;
		glui->sync_live();
	}
	
	// draws silhouette edges via hardware methods
	switch (edge)
	{
		case 0:		// none
			break;
			
		case 1:		// silhouettes only
			drawObjects();
			
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, edgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);
		
			glUseProgramObjectARB(edgeGLSL);

			drawFullsizeQuad();

			glUseProgramObjectARB(0);
			
			glBindTexture(GL_TEXTURE_2D, edgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);

			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
			
			glEnable(GL_TEXTURE_2D);
	
			drawFullsizeQuad();
			
			glDisable(GL_TEXTURE_2D);
			break;
		
		case 2:		// crease edges only
		
			drawObjectsForNormals();

			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, edgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);
						
			glUseProgramObjectARB(edgeGLSL);
			
			drawFullsizeQuad();
			
			glUseProgramObjectARB(0);
		
			glBindTexture(GL_TEXTURE_2D, edgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);			
			
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
			
			glEnable(GL_TEXTURE_2D);
			
			drawFullsizeQuad();
			glDisable(GL_TEXTURE_2D);
			break;
			
		case 3:		// both
			break;
	}

	// draws geometry using image space silhouette methods
	if (lightMode == 0)
		glUseProgramObjectARB(diffuseGLSL);
	else
		glUseProgramObjectARB(toneGLSL);
	
	switch (silhouette)
	{
		case 0:		// translates back facing polygons forward
			glColor3f(1.0, 1.0, 1.0);	
			glCullFace(GL_BACK);
			glDepthFunc(GL_LESS);

			drawObjects();

			glCullFace(GL_FRONT);
			glDepthFunc(GL_LEQUAL);
			glColor3f(0.0, 0.0, 0.0);

			glPushMatrix();
				glTranslatef(eyePos[0] * eyeDistance * k, eyePos[1]  * eyeDistance * k, eyePos[2]  * eyeDistance * k);
				glUseProgramObjectARB(0);
				drawObjects();
			glPopMatrix();
			break;
		case 1:		// draws back facing polygons with larger line size
			glColor3f(0.0, 0.0, 0.0);	
			glCullFace(GL_FRONT);
			glDepthFunc(GL_LESS);
			glPolygonMode(GL_BACK, GL_LINE);
			glLineWidth(3 * borderThickness);
		
			glUseProgramObjectARB(0);

			drawObjects();

			glCullFace(GL_BACK);
			glDepthFunc(GL_LEQUAL);
			glColor3f(1.0, 1.0, 1.0);
		
			glPushMatrix();
				glPolygonMode (GL_BACK, GL_FILL);

					if (lightMode == 0)
						glUseProgramObjectARB(diffuseGLSL);
					else
						glUseProgramObjectARB(toneGLSL);

				drawObjects();	
		glPopMatrix();
		break;
	case 2:		// draws geometry without any silhouette methods
		glCullFace(GL_BACK);
		drawObjects();	
		break;
}
	glUseProgramObjectARB(0);


	myGlutReshape(0,0);
}

/**
*	The main function.  Sets up the GLUT callback functions,
*	and builds the GLUI GUI interface.
*/
int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);
    glutInitWindowSize(WINDOW_WIDTH + viewX, WINDOW_HEIGHT); 
    glutInitWindowPosition (100, 100);
    
	main_window = glutCreateWindow ("Image Precision Silhouette Edges");

	glewInit();
	Init();
	createShaders();

    glutReshapeFunc(myGlutReshape);
	
    glutDisplayFunc(Display); 
	glutMouseFunc(myGlutMouse);
	glutMotionFunc(myGlutMotion);
	glutIdleFunc(myGlutIdle);
	
	glui = GLUI_Master.create_glui_subwindow(main_window, GLUI_SUBWINDOW_LEFT);
	glui->add_button("Quit", 0, (GLUI_Update_CB)exit);	

	borderRollout = glui->add_rollout("Silhouette Control");
	borderPanel = glui->add_panel_to_panel(borderRollout, "", GLUI_PANEL_NONE);
	glui->add_statictext_to_panel(borderPanel, "Image Precision Method");
	borderRadio = glui->add_radiogroup_to_panel(borderPanel, &silhouette);
	glui->add_radiobutton_to_group(borderRadio, "Translation");
	glui->add_radiobutton_to_group(borderRadio, "Line");
	glui->add_radiobutton_to_group(borderRadio, "No Silhouettes");
	borderSpinner = glui->add_spinner_to_panel(borderPanel, "Thickness", GLUI_SPINNER_INT, &borderThickness);
	borderSpinner->set_int_limits(1, 10);
	
	glui->add_statictext_to_panel(borderPanel, "");
	glui->add_statictext_to_panel(borderPanel, "Hardware Extraction");
	edgeRadio = glui->add_radiogroup_to_panel(borderPanel, &edge);
	glui->add_radiobutton_to_group(edgeRadio, "Off");
	glui->add_radiobutton_to_group(edgeRadio, "Silhouettes");
	glui->add_radiobutton_to_group(edgeRadio, "Creases");
	glui->add_radiobutton_to_group(edgeRadio, "Both");
	thresholdSpinner = glui->add_spinner_to_panel(borderPanel, "Threshold", GLUI_SPINNER_FLOAT, &borderThreshold);
	thresholdSpinner->set_float_limits(0.0, 5.0);

	lightRollout = glui->add_rollout("Light Control");
	lightPanel = glui->add_panel_to_panel(lightRollout, "", GLUI_PANEL_NONE);
	lightRadio = glui->add_radiogroup_to_panel(lightPanel, &lightMode);
	glui->add_radiobutton_to_group(lightRadio, "Diffuse Only");
	glui->add_radiobutton_to_group(lightRadio, "NPR Model");
	blue1Spinner = glui->add_spinner_to_panel(lightPanel, "Blue Hue", GLUI_SPINNER_FLOAT, &blue1);
	blue2Spinner = glui->add_spinner_to_panel(lightPanel, "Blue Value", GLUI_SPINNER_FLOAT, &blue2);
	yellow1Spinner = glui->add_spinner_to_panel(lightPanel, "Yellow Hue", GLUI_SPINNER_FLOAT, &yellow1);
	yellow2Spinner = glui->add_spinner_to_panel(lightPanel, "Yellow Value", GLUI_SPINNER_FLOAT, &yellow2);
	blue1Spinner->set_float_limits(0.0, 1.0);
	blue2Spinner->set_float_limits(0.0, 1.0);
	yellow1Spinner->set_float_limits(0.0, 1.0);
	yellow2Spinner->set_float_limits(0.0, 1.0);
	
	eye[0] = 0;
	eye[1] = 0;
	eye[2] = 20;
	lookat[0] = 0;
	lookat[1] = 0;
	lookat[2] = 0;
	
    glutMainLoop();
    
    return 0;
}