#include "./include/main.h"

int cur_button = -1;					// mouse handling for main window
int last_x;								// mouse position
int last_y;								// mouse position
float eye[3];							// camera position
float lookat[3];						// camera look position
int main_window;						// the main window name
int viewX = 209, viewY, viewW, viewH;	// variables to builds the viewport around the GLUI GUI

GLUI				*glui;				// pointers for all of the glui controls
GLUI_RadioGroup		*borderRadio;
GLUI_RadioGroup		*lightRadio;
GLUI_RadioGroup		*edgeRadio;
GLUI_Rollout		*borderRollout;
GLUI_Rollout		*lightRollout;
GLUI_Panel			*lightPanel;
GLUI_Panel			*borderPanel;
GLUI_Spinner		*borderSpinner;
GLUI_Spinner		*thresholdSpinner;
GLUI_Spinner		*blue1Spinner;
GLUI_Spinner		*blue2Spinner;
GLUI_Spinner		*yellow1Spinner;
GLUI_Spinner		*yellow2Spinner;

float blue1		= 0.4;			// relative blue hue
float blue2		= 0.4;			// relative blue value
float yellow1	= 0.2;			// relative yellow hue
float yellow2	= 0.6;			// relative yellow value

GLuint primaryEdgeTex;			// texture for hardware edges
GLuint colorTex;				// texture for hardware fill
GLuint TAM1[6];					// texture for hatches

static unsigned *image;			// pointer to texture data
static int width;				// width of texture data
static int height;				// height of texture data
static int components;			// components of texture data

GLfloat  diffuseLight[] = {1.0, 1.0, 1.0, 1.0};		// color of overall ambient light
GLfloat	 lightPos[]		= {-10.0, 10.0, 0.0, 1.0};	// position of light
float lightIntensity	= 1.0;						// controls intensity of scene light

int silhouette			= 1;	// switch for image space silhouette mode
int edge				= 0;	// switch for hardware silhouette mode
int lightMode			= 1;	// switch for lighting model
int borderThickness		= 1.0;	// switch for image space silhouette thickness
float borderThreshold	= 1.0;	// switch for hardware edge detection threshold

GLenum diffuseGLSL;				// names for GLSL shader program elements
GLenum diffuseVertex;
GLenum diffuseFragment;
GLenum toneGLSL;
GLenum toneVertex;
GLenum toneFragment;
GLenum edgeVertex;
GLenum edgeFragment;
GLenum edgeGLSL;
GLenum normalVertex;
GLenum normalFragment;
GLenum normalGLSL;
GLenum hatchVertex;
GLenum hatchFragment;
GLenum hatchGLSL;

/**
*	Idle function for GLUT callback
 */
void myGlutIdle(void)
{
	if (glutGetWindow() != main_window)
		glutSetWindow(main_window);
				
	glutPostRedisplay();
}

/**
*	Mouse callback function for GLUT
 *
 *	@param	button	the current mouse button
 *	@param	state	the state of 'button'
 *	@param	x		the x coord of the mouse
 *	@param	y		the y coord of the mouse
 */
void myGlutMouse(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN)
		cur_button = button;
	else
	{
		if (button == cur_button)
			cur_button = -1;
	}
	
	last_x = x;
	last_y = y;
}

/**
*	Mouse motion callback function for GLUT
 *
 *	@param	x		the x coord of the mouse
 *	@param	y		the y coord of the mouse
 */
void myGlutMotion(int x, int y)
{
	int dx = x-last_x;			// change in mouse position
	int dy = y-last_y;			// change in mouse position
	
	float scale, len, theta;	// controls camera movement
	float neye[3], neye2[3];	// temp eye positions
	float f[3], r[3], u[3];		// more camera coords
	
	switch(cur_button)
	{
		case GLUT_LEFT_BUTTON:	// translate
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			
			scale = sqrt(v_getVectorLength(f)) * 0.007;	// scale the change by how far away we are
			
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			v_normalize(r);
			v_normalize(u);
			
			eye[0] += -r[0]*dx*scale + u[0]*dy*scale;
			eye[1] += -r[1]*dx*scale + u[1]*dy*scale;
			eye[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			lookat[0] += -r[0]*dx*scale + u[0]*dy*scale;
			lookat[1] += -r[1]*dx*scale + u[1]*dy*scale;
			lookat[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			break;
			
		case GLUT_MIDDLE_BUTTON:	// zoom
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			
			len = v_getVectorLength(f);
			v_normalize(f);
			
			len -= sqrt(len)*dx*0.03;	// scale the change by how far away we are
			
			eye[0] = lookat[0] - len*f[0];
			eye[1] = lookat[1] - len*f[1];
			eye[2] = lookat[2] - len*f[2];
			
			// make sure the eye and lookat points are sufficiently far away
			// push the lookat point forward if it is too close
			if (len < 1)
			{
				lookat[0] = eye[0] + f[0];
				lookat[1] = eye[1] + f[1];
				lookat[2] = eye[2] + f[2];
			}
				break;
			
		case GLUT_RIGHT_BUTTON:	// rotate
			
			neye[0] = eye[0] - lookat[0];
			neye[1] = eye[1] - lookat[1];
			neye[2] = eye[2] - lookat[2];
			
			// first rotate in the x/z plane
			theta = -dx * 0.007;
			neye2[0] = (float)cos(theta)*neye[0] + (float)sin(theta)*neye[2];
			neye2[1] = neye[1];
			neye2[2] =-(float)sin(theta)*neye[0] + (float)cos(theta)*neye[2];
			
			// now rotate vertically
			theta = -dy * 0.007;
			
			f[0] = -neye2[0];
			f[1] = -neye2[1];
			f[2] = -neye2[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			len = v_getVectorLength(f);
			v_normalize(f);
			v_normalize(u);
			
			neye[0] = len * ((float)cos(theta)*f[0] + (float)sin(theta)*u[0]);
			neye[1] = len * ((float)cos(theta)*f[1] + (float)sin(theta)*u[1]);
			neye[2] = len * ((float)cos(theta)*f[2] + (float)sin(theta)*u[2]);
			
			eye[0] = lookat[0] - neye[0];
			eye[1] = lookat[1] - neye[1];
			eye[2] = lookat[2] - neye[2];
			
			break;
	}
	
	last_x = x;
	last_y = y;
	
	glutPostRedisplay();
}

/**
*	Reshape callback function for GLUT
 *
 *	@param	x	the x window coord - unused
 *	@param	y	the y window coord - unused
 */
void myGlutReshape(int	x, int y)
{
	GLUI_Master.get_viewport_area(&viewX, &viewY, &viewW, &viewH);
	glViewport(viewX, viewY, viewW, viewH);
}

/**
*	Loads the source code of a shader program by
 *	copying the entire program into a single string.
 *
 *	@param	filename	the filename and location of the source code
 *	@return		a pointer to the built string
 */
const GLcharARB* loadSource(const char* filename) 
{
	std::string buf;
	std::string line;
	
	std::ifstream in(filename);
	
	if(!in) {
		fprintf(stderr, "Could not find file %s\n", filename);
		exit(EXIT_FAILURE);
	}
	
	while(std::getline(in,line)) {
		buf += line; buf += '\n';
	}
	
	GLcharARB* ret = (GLcharARB*)malloc(buf.size()+1);
	strncpy(ret, buf.c_str(), buf.size());
	ret[buf.size()] = '\0';
	
	return ret;
}

/**
*	Loads the shader source code, compiles the seperate elements,
*	links the vertex and fragment shaders to the GLSL program,
*	and compiles the GLSL program.  Probably not the most efficient
*	way of doing this.
*/
void createShaders()
{
	const GLcharARB* diffuseVertexSource;
	const GLcharARB* diffuseFragmentSource;
	const GLcharARB* toneVertexSource;
	const GLcharARB* toneFragmentSource;
	const GLcharARB* edgeVertexSource;
	const GLcharARB* edgeFragmentSource;
	const GLcharARB* normalVertexSource;
	const GLcharARB* normalFragmentSource;
	const GLcharARB* hatchVertexSource;
	const GLcharARB* hatchFragmentSource;
	GLint status;

	diffuseGLSL		= glCreateProgramObjectARB();
	diffuseVertex	= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	diffuseFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	
	toneGLSL		= glCreateProgramObjectARB();
	toneVertex		= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	toneFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);

	edgeGLSL		= glCreateProgramObjectARB();
	edgeVertex		= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	edgeFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);	
	
	normalGLSL		= glCreateProgramObjectARB();
	normalVertex	= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	normalFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	
	hatchGLSL		= glCreateProgramObjectARB();
	hatchVertex	= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	hatchFragment	= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);	
	
	diffuseVertexSource		= loadSource("./diffuseVertex.vert");
	diffuseFragmentSource	= loadSource("./diffuseFragment.frag");
	toneVertexSource		= loadSource("./toneVertex.vert");
	toneFragmentSource		= loadSource("./toneFragment.frag");
	edgeVertexSource		= loadSource("./edgeVertex.vert");
	edgeFragmentSource		= loadSource("./edgeFragment.frag");
	normalVertexSource		= loadSource("./normalVertex.vert");
	normalFragmentSource	= loadSource("./normalFragment.frag");
	hatchVertexSource		= loadSource("./hatchVertex.vert");
	hatchFragmentSource		= loadSource("./hatchFragment.frag");
	
	glShaderSourceARB(diffuseVertex, 1, &diffuseVertexSource, NULL);
	glShaderSourceARB(diffuseFragment, 1, &diffuseFragmentSource, NULL);
	glShaderSourceARB(toneVertex, 1, &toneVertexSource, NULL);
	glShaderSourceARB(toneFragment, 1, &toneFragmentSource, NULL);
	glShaderSourceARB(edgeVertex, 1, &edgeVertexSource, NULL);
	glShaderSourceARB(edgeFragment, 1, &edgeFragmentSource, NULL);
	glShaderSourceARB(normalVertex,		1, &normalVertexSource, NULL);
	glShaderSourceARB(normalFragment,	1, &normalFragmentSource, NULL);
	glShaderSourceARB(hatchVertex,		1, &hatchVertexSource, NULL);
	glShaderSourceARB(hatchFragment,	1, &hatchFragmentSource, NULL);
	
	delete 	diffuseVertexSource;
	delete diffuseFragmentSource;
	delete toneVertexSource;
	delete toneFragmentSource;
	delete edgeVertexSource;
	delete edgeFragmentSource;
	delete normalVertexSource;
	delete normalFragmentSource;
	delete hatchVertexSource;
	delete hatchFragmentSource;
	
	glCompileShaderARB(diffuseVertex);
	glGetObjectParameterivARB(diffuseVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(diffuseVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(diffuseVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(diffuseFragment);
	glGetObjectParameterivARB(diffuseFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(diffuseFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(diffuseFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(diffuseGLSL, diffuseVertex);
	glAttachObjectARB(diffuseGLSL, diffuseFragment);
	
	glLinkProgramARB(diffuseGLSL);
	glGetObjectParameterivARB(diffuseGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(diffuseGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(diffuseGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	glCompileShaderARB(toneVertex);
	glGetObjectParameterivARB(toneVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(toneVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(toneVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(toneFragment);
	glGetObjectParameterivARB(toneFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(toneFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(toneFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(toneGLSL, toneVertex);
	glAttachObjectARB(toneGLSL, toneFragment);
	
	glLinkProgramARB(toneGLSL);
	glGetObjectParameterivARB(toneGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(toneGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(toneGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	glCompileShaderARB(edgeVertex);
	glGetObjectParameterivARB(edgeVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(edgeVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(edgeVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(edgeFragment);
	glGetObjectParameterivARB(edgeFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(edgeFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(edgeFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(edgeGLSL, edgeVertex);
	glAttachObjectARB(edgeGLSL, edgeFragment);
	
	glLinkProgramARB(edgeGLSL);
	glGetObjectParameterivARB(edgeGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(edgeGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(edgeGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	glCompileShaderARB(normalVertex);
	glGetObjectParameterivARB(normalVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(normalVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(normalVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(normalFragment);
	glGetObjectParameterivARB(normalFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(normalFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(normalFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(normalGLSL, normalVertex);
	glAttachObjectARB(normalGLSL, normalFragment);
	
	glLinkProgramARB(normalGLSL);
	glGetObjectParameterivARB(normalGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(normalGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(normalGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	glCompileShaderARB(hatchVertex);
	glGetObjectParameterivARB(hatchVertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(hatchVertex, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(hatchVertex, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(hatchFragment);
	glGetObjectParameterivARB(hatchFragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(hatchFragment, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(hatchFragment, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glAttachObjectARB(hatchGLSL, hatchVertex);
	glAttachObjectARB(hatchGLSL, hatchFragment);
	
	glLinkProgramARB(hatchGLSL);
	glGetObjectParameterivARB(hatchGLSL, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(hatchGLSL, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(hatchGLSL, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
}

/**
*	Initializes the OpenGL state.  Loads the textures
*	and builds display lists.
*/
GLvoid Init(GLvoid)
{
	GLfloat  ambientLight[]			= {0.1, 0.1, 0.1, 0.0};		// color of overall ambient light

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	glEnable(GL_LINE_SMOOTH);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_CULL_FACE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glEnable(GL_LIGHT0);

	glClearColor(0.5, 0.5, 0.5, 0.0);
	glClearDepth(1.0);
	
	glGenTextures(1, &colorTex);
	glBindTexture(GL_TEXTURE_2D, colorTex);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WINDOW_WIDTH, WINDOW_HEIGHT, 0, GL_RGB, GL_FLOAT, NULL);
	
	glGenTextures(1, &primaryEdgeTex);
	glBindTexture(GL_TEXTURE_2D, primaryEdgeTex);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WINDOW_WIDTH, WINDOW_HEIGHT, 0, GL_RGB, GL_FLOAT, NULL);
	
	glGenTextures(6, TAM1);
	glBindTexture(GL_TEXTURE_2D, TAM1[0]);
		image = read_texture("./TAM1-1.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, TAM1[1]);
		image = read_texture("./TAM2-1.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);	
	glBindTexture(GL_TEXTURE_2D, TAM1[2]);
		image = read_texture("./TAM3-1.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, TAM1[3]);
		image = read_texture("./TAM4-1.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, TAM1[4]);
		image = read_texture("./TAM5-1.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, TAM1[5]);
		image = read_texture("./TAM6-1.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, primaryEdgeTex);
	
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, colorTex);
	
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, TAM1[0]);
	
	glActiveTexture(GL_TEXTURE3);
	glBindTexture(GL_TEXTURE_2D, TAM1[1]);
	
	glActiveTexture(GL_TEXTURE4);
	glBindTexture(GL_TEXTURE_2D, TAM1[2]);
	
	glActiveTexture(GL_TEXTURE5);
	glBindTexture(GL_TEXTURE_2D, TAM1[3]);
	
	glActiveTexture(GL_TEXTURE6);
	glBindTexture(GL_TEXTURE_2D, TAM1[4]);
	
	glActiveTexture(GL_TEXTURE7);
	glBindTexture(GL_TEXTURE_2D, TAM1[5]);
}

/**
*	Draws the objects according to the used lighting method.
*	Defaults to drawing each object with a different color.
*
*	@param useNormals	draws the objects using the different shader parameters
*/
GLvoid drawObjects(int drawMode = 0)
{
	GLfloat yellow[]		= {0.8, 0.8, 0.2, 1.0};
	GLfloat green[]			= {0.2, 0.8, 0.2, 1.0};
	GLfloat red[]			= {0.8, 0.2, 0.2, 1.0};
	GLfloat white[]			= {1.0, 1.0, 1.0, 1.0};

	if(drawMode == 1)	
		glUseProgramObjectARB(normalGLSL);

	if(lightMode != 2)
	{
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, yellow);

		glPushMatrix();
		glTranslatef(0.0, -2.6, 0.0);
		glScalef(2.0, 2.0, 2.0);
		glRotatef(45.0, 0.0, 1.0, 0.0);

		glutSolidTorus(0.2, 1.0, 50, 50);
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, green);
		glutSolidTorus(0.2, 1.0, 50, 50);
	
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, red);
		glutSolidTorus(0.2, 1.0, 50, 50);
		
		glTranslatef(0.0, 1.3, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, white);
		glutSolidTorus(0.2, 1.0, 50, 50);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(6.0, 0.0, 0.0);
		glScalef(3.0, 3.0, 3.0);

		glutSolidSphere(1.0, 60.0, 60.0);		
		glTranslatef(-4.0, 0.0, 0.0);

		glutSolidIcosahedron();

		glPopMatrix();
	}
	else
	{
		glPushMatrix();
		glScalef(2.0, 2.0, 2.0);
		glTranslatef(-1.0, 1.0, 0.0);
		
		glDisable(GL_CULL_FACE);
		GLUquadricObj *cylinder;
		cylinder = gluNewQuadric();
		glRotatef(90.0, 1.0, 0.0, 0.0);
		gluQuadricDrawStyle(cylinder, GLU_FILL);
		gluQuadricTexture(cylinder, GL_TRUE);
		gluQuadricNormals(cylinder, GLU_SMOOTH);
		gluCylinder(cylinder, 1.0, 1.0, 2.0, 30.0, 30.0);
		glEnable(GL_CULL_FACE);

		glTranslatef(3.0, -1.0, 0.0);

		GLUquadricObj *sphere;
		sphere = gluNewQuadric();
		gluQuadricDrawStyle(sphere, GLU_FILL);
		gluQuadricTexture(sphere, GL_TRUE);
		gluQuadricNormals(sphere, GLU_SMOOTH);
		gluSphere(sphere, 1.0, 30.0, 30.0);
		glPopMatrix();
	}

}

/**
*	Initializes the uniform variables for the shader programs.
*/
GLvoid setUniformVars(GLvoid)
{
	glUseProgramObjectARB(toneGLSL);
	
		GLint handle = glGetUniformLocationARB(toneGLSL, "blueShift");
		glUniform1fARB(handle, blue1);
		
		handle = glGetUniformLocationARB(toneGLSL, "blueLuminance");
		glUniform1fARB(handle, blue2);
		
		handle = glGetUniformLocationARB(toneGLSL, "yellowShift");
		glUniform1fARB(handle, yellow1);

		handle = glGetUniformLocationARB(toneGLSL, "yellowLuminance");
		glUniform1fARB(handle, yellow2);

	glUseProgramObjectARB(edgeGLSL);
	
		handle = glGetUniformLocationARB(edgeGLSL, "threshold");
		glUniform1fARB(handle, borderThreshold);
		
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, primaryEdgeTex);
		handle = glGetUniformLocationARB(edgeGLSL, "primaryUnit");
		glUniform1iARB(handle, 0);
		
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, colorTex);	
		handle = glGetUniformLocationARB(edgeGLSL, "colorUnit");
		glUniform1iARB(handle, 1);
	
	glUseProgramObjectARB(hatchGLSL);
	
		glActiveTexture(GL_TEXTURE2);
		glBindTexture(GL_TEXTURE_2D, TAM1[0]);
		handle = glGetUniformLocationARB(hatchGLSL, "hatch0");
		glUniform1iARB(handle, 2);

		glActiveTexture(GL_TEXTURE3);
		glBindTexture(GL_TEXTURE_2D, TAM1[1]);
		handle = glGetUniformLocationARB(hatchGLSL, "hatch1");
		glUniform1iARB(handle, 3);
			
		glActiveTexture(GL_TEXTURE4);
		glBindTexture(GL_TEXTURE_2D, TAM1[2]);
		handle = glGetUniformLocationARB(hatchGLSL, "hatch2");
		glUniform1iARB(handle, 4);
			
		glActiveTexture(GL_TEXTURE5);
		glBindTexture(GL_TEXTURE_2D, TAM1[3]);
		handle = glGetUniformLocationARB(hatchGLSL, "hatch3");
		glUniform1iARB(handle, 5);
			
		glActiveTexture(GL_TEXTURE6);
		glBindTexture(GL_TEXTURE_2D, TAM1[4]);
		handle = glGetUniformLocationARB(hatchGLSL, "hatch4");
		glUniform1iARB(handle, 6);
		
		glActiveTexture(GL_TEXTURE7);
		glBindTexture(GL_TEXTURE_2D, TAM1[5]);
		handle = glGetUniformLocationARB(hatchGLSL, "hatch5");
		glUniform1iARB(handle, 7);
	
	glUseProgramObjectARB(0);
}

/**
*	Draws a full sized quad to force shader computation
*	or to display full screen results of rendering.
*/
GLvoid drawFullsizeQuad(GLvoid)
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);		glVertex3f(-1.0, -1.0, 0.0);
		glTexCoord2f(1.0, 0.0);		glVertex3f(1.0, -1.0, 0.0);
		glTexCoord2f(1.0, 1.0);		glVertex3f(1.0, 1.0, 0.0);
		glTexCoord2f(0.0, 1.0);		glVertex3f(-1.0, 1.0, 0.0);
	glEnd();
	
	glPopMatrix();
	
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

/**
*	Draws multiple instances of the geometry, depending on the
*	desired render status.  See report for more details.
*/
void Display(void)
{	
	vector3 eyePos;									// position of camera
	float eyeDistance = v_distance(eye, lookat);	// distance from the camera to the lookat point
	float k = 0.01 * borderThickness;				// thickness of silhouette
	
	eyePos[0] = eye[0];
	eyePos[1] = eye[1];
	eyePos[2] = eye[2];
	v_normalize(eyePos);

	// set up projection and modelview matrix
	diffuseLight[0] = diffuseLight[1] = diffuseLight[2] = lightIntensity;
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluPerspective(70.0, 1.0, 1.0, 40.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	
	gluLookAt(eye[0], eye[1], eye[2], lookat[0], lookat[1], lookat[2], 0, 1, 0);

	glUseProgramObjectARB(toneGLSL);
	
	GLint handle = glGetUniformLocationARB(toneGLSL, "blueShift");
	glUniform1fARB(handle, blue1);
	
	handle = glGetUniformLocationARB(toneGLSL, "blueLuminance");
	glUniform1fARB(handle, blue2);
	
	handle = glGetUniformLocationARB(toneGLSL, "yellowShift");
	glUniform1fARB(handle, yellow1);
	
	handle = glGetUniformLocationARB(toneGLSL, "yellowLuminance");
	glUniform1fARB(handle, yellow2);
	

	glUseProgramObjectARB(edgeGLSL);
	handle = glGetUniformLocationARB(edgeGLSL, "threshold");
	glUniform1fARB(handle, borderThreshold);

	// draws the objects initially to fill colorTex texture
	// uses lighting model selected
	if (lightMode == 0)
		glUseProgramObjectARB(diffuseGLSL);
	else if (lightMode == 1)
		glUseProgramObjectARB(toneGLSL);
	else
		glUseProgramObjectARB(hatchGLSL);
	
	drawObjects();
	
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, colorTex);
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);
	
	glUseProgramObjectARB(0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (edge != 0)
	{
		silhouette = 2;
		glui->sync_live();
	}

	// draws silhouette edges via hardware methods	
	switch (edge)
	{
		case 0:		// none
			break;
		case 1:		// silhouette only
			drawObjects();
				
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, primaryEdgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);
		
			glUseProgramObjectARB(edgeGLSL);

			drawFullsizeQuad();

			glUseProgramObjectARB(0);
			break;
		case 2:		// crease only
			drawObjects(1);
			
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, primaryEdgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);
						
			glUseProgramObjectARB(edgeGLSL);
			
			drawFullsizeQuad();
			
			glUseProgramObjectARB(0);
						
			break;
		case 3:		// both
		
			drawObjects(1);
			
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, primaryEdgeTex);
			glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, viewX, viewY, viewW, viewH);
			
			glUseProgramObjectARB(edgeGLSL);
			
			drawFullsizeQuad();
			
			glUseProgramObjectARB(0);
			break;
	}

	// draws geometry using image space silhouette methods
	if (lightMode == 0)
		glUseProgramObjectARB(diffuseGLSL);
	else if (lightMode == 1)
		glUseProgramObjectARB(toneGLSL);
	else
		glUseProgramObjectARB(hatchGLSL);
	
	switch (silhouette)
	{
		case 0:		// translates back facing polygons forward
			glColor3f(1.0, 1.0, 1.0);	
			glCullFace(GL_BACK);
			glDepthFunc(GL_LESS);
			drawObjects();
			glCullFace(GL_FRONT);
			glDepthFunc(GL_LEQUAL);
			glColor3f(0.0, 0.0, 0.0);	
			glPushMatrix();
				glTranslatef(eyePos[0] * eyeDistance * k, eyePos[1]  * eyeDistance * k, eyePos[2]  * eyeDistance * k);
				glUseProgramObjectARB(0);

				drawObjects();
				glCullFace(GL_BACK);
			glPopMatrix();
			break;
		case 1:			// draws back facing polygons with larger line size
			glColor3f(0.0, 0.0, 0.0);	
			glCullFace(GL_FRONT);
			glDepthFunc(GL_LESS);
			glPolygonMode(GL_BACK, GL_LINE);
			glLineWidth(3 * borderThickness);
			
			glUseProgramObjectARB(0);
			drawObjects();
			glCullFace(GL_BACK);
			glDepthFunc(GL_LEQUAL);
			glColor3f(1.0, 1.0, 1.0);
			
			glPushMatrix();
				glPolygonMode (GL_BACK, GL_FILL);

			if (lightMode == 0)
				glUseProgramObjectARB(diffuseGLSL);
			else if (lightMode == 1)
				glUseProgramObjectARB(toneGLSL);
			else
				glUseProgramObjectARB(hatchGLSL);

				drawObjects();
			glPopMatrix();
			break;
		case 2:			// draws geometry without any silhouette methods
			glCullFace(GL_BACK);
			drawObjects();
			break;
	}
	glUseProgramObjectARB(0);

	glutSwapBuffers();
	myGlutReshape(0,0);
}

/**
*	The main function.  Sets up the GLUT callback functions,
*	and builds the GLUI GUI interface.
*/
int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);
    glutInitWindowSize(WINDOW_WIDTH + viewX, WINDOW_HEIGHT); 
    glutInitWindowPosition (100, 100);
    
	main_window = glutCreateWindow ("Image Precision Silhouette Edges");
	glewInit();
	Init();
	createShaders();
	setUniformVars();
	
    glutReshapeFunc(myGlutReshape);
	
    glutDisplayFunc(Display); 
	glutMouseFunc(myGlutMouse);
	glutMotionFunc(myGlutMotion);
	glutIdleFunc(myGlutIdle);
	
	glui = GLUI_Master.create_glui_subwindow(main_window, GLUI_SUBWINDOW_LEFT);
	glui->add_button("Quit", 0, (GLUI_Update_CB)exit);	

	borderRollout = glui->add_rollout("Silhouette Control");
	borderPanel = glui->add_panel_to_panel(borderRollout, "", GLUI_PANEL_NONE);
	glui->add_statictext_to_panel(borderPanel, "Image Precision Method");
	borderRadio = glui->add_radiogroup_to_panel(borderPanel, &silhouette);
	glui->add_radiobutton_to_group(borderRadio, "Translation");
	glui->add_radiobutton_to_group(borderRadio, "Line");
	glui->add_radiobutton_to_group(borderRadio, "No Silhouettes");
	borderSpinner = glui->add_spinner_to_panel(borderPanel, "Thickness", GLUI_SPINNER_INT, &borderThickness);
	borderSpinner->set_int_limits(1, 10);
	
	glui->add_statictext_to_panel(borderPanel, "");
	glui->add_statictext_to_panel(borderPanel, "Hardware Extraction");
	edgeRadio = glui->add_radiogroup_to_panel(borderPanel, &edge);
	glui->add_radiobutton_to_group(edgeRadio, "Off");
	glui->add_radiobutton_to_group(edgeRadio, "Silhouettes");
	glui->add_radiobutton_to_group(edgeRadio, "Creases");
//	glui->add_radiobutton_to_group(edgeRadio, "Both");
	thresholdSpinner = glui->add_spinner_to_panel(borderPanel, "Threshold", GLUI_SPINNER_FLOAT, &borderThreshold);
	thresholdSpinner->set_float_limits(0.0, 5.0);

	lightRollout = glui->add_rollout("Light Control");
	lightPanel = glui->add_panel_to_panel(lightRollout, "", GLUI_PANEL_NONE);
	lightRadio = glui->add_radiogroup_to_panel(lightPanel, &lightMode);
	glui->add_radiobutton_to_group(lightRadio, "Diffuse Only");
	glui->add_radiobutton_to_group(lightRadio, "NPR Model");
	glui->add_radiobutton_to_group(lightRadio, "Hatching");
	blue1Spinner = glui->add_spinner_to_panel(lightPanel, "Blue Temp", GLUI_SPINNER_FLOAT, &blue1);
	blue2Spinner = glui->add_spinner_to_panel(lightPanel, "Blue Strength", GLUI_SPINNER_FLOAT, &blue2);
	yellow1Spinner = glui->add_spinner_to_panel(lightPanel, "Yellow Temp", GLUI_SPINNER_FLOAT, &yellow1);
	yellow2Spinner = glui->add_spinner_to_panel(lightPanel, "Yellow Strength", GLUI_SPINNER_FLOAT, &yellow2);
	blue1Spinner->set_float_limits(0.0, 1.0);
	blue2Spinner->set_float_limits(0.0, 1.0);
	yellow1Spinner->set_float_limits(0.0, 1.0);
	yellow2Spinner->set_float_limits(0.0, 1.0);

	eye[0] = 0;
	eye[1] = 0;
	eye[2] = 20;
	lookat[0] = 0;
	lookat[1] = 0;
	lookat[2] = 0;
	
    glutMainLoop();
    
    return 0;
}

